-- Puffin: A Tojaso FowlPlay Project
-- LibDataBroker component

local Puffin_LDB = {}
local LQT = nil
local menuBroker, clockBroker, systemBroker, infoBroker, switchBroker
local updateFrame = CreateFrame("Frame")
local tc = 0
local startTime = GetTime()
local activeTooltip, scanTooltip
local slots = { "Head", "Shoulder", "Chest", "Waist", "Legs", "Feet", "Wrist", "Hands", "MainHand", "SecondaryHand", }
local gridFrame = nil
local cycle = 0
local horiz, vert = 64, 40 -- must be multiples of 4, should match aspect ratio of the display for best results

function Puffin_LDB.OnInitialize()
	LoadAddOn("LibDataBroker-1.1")
	LQT = LibStub("LibQTip-1.0")
	scanTooltip = CreateFrame("GameTooltip", "Puffin_ScanTip", nil, "GameTooltipTemplate")
	scanTooltip:SetOwner(UIParent, "ANCHOR_NONE")
end

local function HideTooltip(tooltip)
	if not MouseIsOver(tooltip) then
		local frame = tooltip._activeFrame
		if frame and frame.tooltip then
			tooltip._activeFrame = nil
			tooltip:SetScript('OnLeave', nil)
			LibQTip:Release(tooltip)
			frame.tooltip = nil; activeTooltip = nil
		end
	end
end

local function AddLine(tooltip, text, f, noCombat)
	if noCombat and InCombatLockdown() then
		tooltip:AddLine(string.format("|c80808000%s|r", text))
	else
		local line = tooltip:AddLine(text)
		tooltip:SetLineScript(line, "OnMouseUp", f)
	end
end

local function GetTimeText(zone, military, ampm)
	local t = ""
	local h, m = GetGameTime()
	if zone == "local" then local d = date("*t"); h = d.hour end
	if zone == "session" then
		local s = GetTime() - startTime; h = math.floor(s / 3600); m = math.floor((s - (h * 3600)) / 60)
		if h > 0 then t = string.format("%d:%02d:%02d", h, m, math.floor(s - (h * 3600) - (m * 60)))
		else t = string.format("%d:%02d", m, math.floor(s - (m * 60))) end
	else	
		if military then
			t = string.format ("%02d:%02d", h, m)
		else
			if ampm then ampm = " AM" end
			if h >= 12 then h = h - 12; if ampm then ampm = " PM" end end
			if h == 0 then h = 12 end
			t = string.format ("%d:%02d%s", h, m, ampm or "")
		end
	end
	return t
end

local function DisplayGridPattern()
	cycle = cycle + 1 -- alternate between no grid, big grid, small grid
	if cycle == 3 then cycle = 0 end
	if gridFrame then gridFrame:Hide(); gridFrame = nil end

	if cycle > 0 then -- display either red or green grid
		gridFrame = CreateFrame('Frame', nil, UIParent) 
		gridFrame:SetAllPoints(UIParent)
		local w = GetScreenWidth() / horiz
		local r, g = 0, 1
		if cycle == 2 then r = 1; g = 0 end
		for i = 0, horiz do
			local t = gridFrame:CreateTexture(nil, 'BACKGROUND')
			if i == (horiz / 2) then t:SetTexture(0, 1, 1, 0.5) else t:SetTexture(r, g, 0, 0.25) end
			t:SetPoint('TOPLEFT', gridFrame, 'TOPLEFT', i * w - 0.5, 0)
			t:SetPoint('BOTTOMRIGHT', gridFrame, 'BOTTOMLEFT', i * w + 0.5, 0)
		end
		local h = GetScreenHeight() / vert
		for i = 0, vert do
			local t = gridFrame:CreateTexture(nil, 'BACKGROUND')
			if i == (vert / 2) then t:SetTexture(0, 1, 1, 0.5) else t:SetTexture(r, g, 0, 0.25) end
			t:SetPoint('TOPLEFT', gridFrame, 'TOPLEFT', 0, -i * h + 0.5)
			t:SetPoint('BOTTOMRIGHT', gridFrame, 'TOPRIGHT', 0, -i * h - 0.5)
		end	
	end
end

updateFrame:SetScript("OnUpdate", function(self, e)
	tc = tc + e; if tc > 1 then
		tc = 0
		if clockBroker then clockBroker.text = GetTimeText("local", false, true) end
		if systemBroker then systemBroker.text = string.format("%d fps", math.floor(GetFramerate())) end
		if infoBroker then if HasNewMail() then infoBroker.icon = GetItemIcon(5735) else infoBroker.icon = GetItemIcon(31823) end end
		if activeTooltip then HideTooltip(activeTooltip) end
	end
end)

local function format_int(number)
	if not number then number = 0 end
	if math.abs(number) < 1000 then return number end
	local i, j, minus, int, fraction = tostring(number):find('([-]?)(%d+)([.]?%d*)')
	int = int:reverse():gsub("(%d%d%d)", "%1,") -- reverse the int-string and append a comma to all blocks of 3 digits
	return minus .. int:reverse():gsub("^,", "") .. fraction -- reverse int-string, remove optional commas, restore minus and fractional part
end

function Puffin_LDB.OnEnable()
	local LDB = LibStub("LibDataBroker-1.1", true)
	if LDB then
		menuBroker = LDB:NewDataObject("Puffin Menu", {
			type = "launcher", text = "Menu", label = "Menu", icon = [[Interface\AddOns\Puffin\Images\Puffin]],
			OnEnter = function(frame)
				if LQT then
					local tooltip = LQT:Acquire("Puffin_MenuTooltip", 1, "LEFT")
					frame.tooltip = tooltip; activeTooltip = tooltip
					tooltip:AddHeader("|cffffcc00Puffin Menu|r")
					AddLine(tooltip, CHARACTER_BUTTON, function() securecall(ToggleCharacter, "PaperDollFrame") end)			
					AddLine(tooltip, SPELLBOOK_ABILITIES_BUTTON, function()
							if not InCombatLockdown() then ToggleSpellBook(BOOKTYPE_SPELL) end
						end, true)			
					AddLine(tooltip, TALENTS_BUTTON, function()
							if (not PlayerTalentFrame) then LoadAddOn('Blizzard_TalentUI') end
							if (not GlyphFrame) then LoadAddOn('Blizzard_GlyphUI') end 
							PlayerTalentFrame_Toggle()
						end )			
					AddLine(tooltip, ACHIEVEMENT_BUTTON, function() securecall(ToggleAchievementFrame) end)			
					AddLine(tooltip, QUESTLOG_BUTTON, function() securecall(ToggleFrame, QuestLogFrame) end)			
					AddLine(tooltip, GUILD, function() ToggleGuildFrame() end)			
					AddLine(tooltip, SOCIAL_BUTTON, function() securecall(ToggleFriendsFrame, 1) end)			
					AddLine(tooltip, PLAYER_V_PLAYER, function() securecall(ToggleFrame, PVPFrame) end)			
					AddLine(tooltip, DUNGEONS_BUTTON, function() securecall(ToggleLFDParentFrame) end)			
					AddLine(tooltip, MOUNTS_AND_PETS, function() if not InCombatLockdown() then TogglePetJournal() end end, true)			
					AddLine(tooltip, ENCOUNTER_JOURNAL, function() securecall(ToggleEncounterJournal) end)			
					AddLine(tooltip, "Calendar", function() securecall(ToggleCalendar) end)			
					AddLine(tooltip, "Time Manager", function() securecall(ToggleTimeManager) end)			
					AddLine(tooltip, GM_EMAIL_NAME, function() securecall(ToggleHelpFrame) end)			
					tooltip:SmartAnchorTo(frame)
					tooltip:Show()
				end
			end,
			OnLeave = function(frame)
				if LQT then
					if frame.tooltip then
						frame.tooltip._activeFrame = frame
						if MouseIsOver(frame.tooltip) then
							frame.tooltip:SetScript("OnLeave", HideTooltip)
						else
							HideTooltip(frame.tooltip)
						end
					end
				end
			end
		})

		clockBroker = LDB:NewDataObject("Puffin Clock", {
			type = "data source", text = "Clock", label = "Clock", icon = GetItemIcon(4389),
			OnEnter = function(frame)
				if LQT then
					local tooltip = LQT:Acquire("Puffin_ClockTooltip", 1, "LEFT")
					frame.tooltip = tooltip
					local _, _, clockIcon = GetSpellInfo(129772)
					tooltip:AddHeader("|cffffcc00Puffin Clock|r")
					tooltip:AddLine(string.format("%s Time (Local)", GetTimeText("local", false, true)))
					tooltip:AddLine(string.format("%s Time (Server)", GetTimeText("server", false, true)))
					tooltip:AddLine(string.format("|T%s:16|t |cffffcc00%s Session|r", clockIcon, GetTimeText("session", false, false)))
					tooltip:SmartAnchorTo(frame)
					tooltip:Show()
				end
			end,
			OnLeave = function(frame) if LQT then LibQTip:Release(frame.tooltip) end end
		})
		
		infoBroker = LDB:NewDataObject("Puffin Info", {
			type = "data source", text = "Info", label = "Info", icon = GetItemIcon(31823),
			OnEnter = function(frame)
				if LQT then
					local tooltip = LQT:Acquire("Puffin_InfoTooltip", 1, "LEFT")
					frame.tooltip = tooltip
					local repairCost, durability = 0, 1
					for _, slot in ipairs(slots) do
						local id = GetInventorySlotInfo(slot .. "Slot")
						local hasItem, _ ,cost = scanTooltip:SetInventoryItem("player", id)
						if hasItem then repairCost = repairCost + cost end
					end
					for slotId = 1, 18 do
						if GetInventoryItemLink("player", slotId) then
							local x, y = GetInventoryItemDurability(slotId)
							if x and y > 0 then if x / y < durability then durability = x / y end end
						end
					end
					tooltip:AddHeader("|cffffcc00Puffin Info|r")
					if UnitLevel("player") < GetMaxPlayerLevel() then
						local xp, xpMax = UnitXP("player"), UnitXPMax("player")
						tooltip:AddLine(string.format("XP %d%% %s / %s (%s Rested)", xp * 100 / xpMax,
							format_int(xp), format_int(xpMax), format_int(GetXPExhaustion())))
					end
					tooltip:AddLine(string.format("Durability %d%% (%s Repair Cost)", durability * 100, GetCoinTextureString(repairCost)))
					tooltip:AddLine(GetCoinTextureString(GetMoney()))
					if HasNewMail() then tooltip:AddLine("|cffffcc00You have mail!|r") end
					tooltip:SmartAnchorTo(frame)
					tooltip:Show()
				end
			end,
			OnLeave = function(frame) if LQT then LibQTip:Release(frame.tooltip) end end
		})
		
		systemBroker = LDB:NewDataObject("Puffin System", {
			type = "data source", text = "System", label = "System", icon = GetItemIcon(89990),
			OnEnter = function(frame)
				if LQT then
					local tooltip = LQT:Acquire("Puffin_SystemTooltip", 1, "LEFT")
					frame.tooltip = tooltip
					local _, _, latencyHome, latencyWorld = GetNetStats()
					local version, internalVersion = GetBuildInfo()
					local _, _, wowIcon = GetSpellInfo(100951)
					local mem, addons = 0, 0
					for i = 1, GetNumAddOns() do
						local _, _, _, enabled = GetAddOnInfo(i)
						if enabled then addons = addons + 1; mem = mem + GetAddOnMemoryUsage(i) end
					end
					local resolution, api = GetCVar("gxResolution"), GetCVar("gxApi")
					if api == "D3D11" then api = "DX11" elseif api == "D3D9" then api = "DX9" end
					tooltip:AddHeader("|cffffcc00Puffin System|r")
					tooltip:AddLine(string.format("%d ms Latency (Home)", math.floor(latencyHome or 0)))
					tooltip:AddLine(string.format("%d ms Latency (World)", math.floor(latencyWorld or 0)))
					tooltip:AddLine(string.format("%d Addons Enabled (%0.2f MB)", addons, mem / 1000))
					tooltip:AddLine(string.format("%s Graphics (%s)", api, resolution))
					tooltip:AddLine(string.format("|T%s:16|t |cffffcc00%s Build %d|r", wowIcon, version, internalVersion))
					tooltip:SmartAnchorTo(frame)
					tooltip:Show()
				end
			end,
			OnLeave = function(frame) if LQT then LibQTip:Release(frame.tooltip) end end
		})
		
		switchBroker = LDB:NewDataObject("Switcher", {
			type = "launcher", text = "Switcher", icon = "Interface\\Icons\\Spell_Arcane_TeleportThunderBluff",
			OnClick = function(_, msg)
				if msg == "LeftButton" then
					if IsShiftKeyDown() then
						local maxMode = 1 - GetCVar("gxMaximize")
						SetCVar("gxMaximize", maxMode); SetCVar("gxWindow", 1)
					else
						local screenMode = 1 - GetCVar("gxWindow")
						SetCVar("gxMaximize", 0); SetCVar("gxWindow", screenMode)
					end
					GameTooltip:Hide(); RestartGx(); GameTooltip:Show()
				elseif msg == "RightButton" then
					if IsShiftKeyDown() then DisplayGridPattern() else GameTooltip:Hide(); Screenshot(); GameTooltip:Show() end
				end
			end,
			OnTooltipShow = function(tooltip)
				if GetCVar("gxWindow") == "1" then
					if GetCVar("gxMaximize") == "1" then
						tooltip:AddDoubleLine("Switcher", "Windowed (Full Screen)")
					else
						tooltip:AddDoubleLine("Switcher", "Windowed")
					end
				else
					tooltip:AddDoubleLine("Switcher", "Full Screen")
				end
				tooltip:AddLine("|cffffff00" .. "Left-click|r to toggle between full screen and windowed")
				tooltip:AddLine("|cffffff00" .. "Shift-left-click|r select windowed (full screen)")
				tooltip:AddLine("|cffffff00" .. "Right-click|r to take a screenshot")
				tooltip:AddLine("|cffffff00" .. "Shift-right-click|r to toggle display grid overlay")
				tooltip:Show()
			end,
		})
	end
end

Puffin.RegisterComponent("LDB", Puffin_LDB)
