-- Puffin: A Tojaso FowlPlay Project
-- Model component for maintaining spell information tables

local Model_Spells = {}
local DB = nil
local localSpells = {} -- map from local spell names to English spell names

local runeSpells = {
	["Army of the Dead"] = { blood = true, frost = true, unholy = true, id = 42650 },
	["Blood Boil"] = { blood = true, id = 48721 },
	["Blood Strike"] = { blood = true, id = 45902 },
	["Chains of Ice"] = { frost = true, id = 45524 },
	["Control Undead"] = { unholy = true, id = 111673 },
	["Dark Transformation"] = { unholy = true, id = 63560 },
	["Death and Decay"] = { unholy = true, id = 43265 },
	["Death Gate"] = { unholy = true, id = 50977 },
	["Death Strike"] = { frost = true, unholy = true, id = 49998 },
	["Festering Strike"] = { blood = true, frost = true, id = 85948 },
	["Heart Strike"] = { blood = true, id = 55050 },
	["Howling Blast"] = { frost = true, id = 49184 },
	["Icy Touch"] = { frost = true, id = 45477 },
	["Obliterate"] = { frost = true, unholy = true, id = 49020 },
	["Path of Frost"] = { frost = true, id = 3714 },
	["Pestilence"] = { blood = true, id = 50842 },
	["Pillar of Frost"] = { frost = true, id = 51271 },
	["Plague Strike"] = { unholy = true, id = 45462 },
	["Rune Tap"] = { blood = true, id = 48982 },
	["Scourge Strike"] = { unholy = true, id = 55090 },
	["Soul Reaper"] = { frost = true, id = 130735 },
	["Strangulate"] = { blood = true, id = 47476 },
}

local lockSpells = {
	["DEATHKNIGHT"] = {
		["Death Coil"] = { school = "Shadow", id = 47541 },
	},
	["DRUID"] = {
		["Rejuvenation"] = { school = "Nature", id = 774 },
	},
	["HUNTER"] = {
		["Arcane Shot"] = { school = "Arcane", id = 3044 },
		["Serpent Sting"] = { school = "Nature", id = 1978 },
	},
	["MAGE"] = {
		["Arcane Explosion"] = { school = "Arcane", id = 1449 }, 
		["Fireball"] = { school = "Fire", id = 133 }, 
		["Frostbolt"] = { school = "Frost", id = 116 }, 
		["Ice Lance"] = { school = "Frost", id = 30455 }, 
		["Molten Armor"] = { school = "Fire", id = 30482 }, 
	},
	["MONK"] = {
		["Resuscitate"] = { school = "Nature", id = 115178 }, 
		["Soothing Mist"] = { school = "Nature", id = 115175 },
	},
	["PALADIN"] = {
		["Flash of Light"] = { school = "Holy", id = 19750 },
	},
	["PRIEST"] = {
		["Shadow Word: Pain"] = { school = "Shadow", id = 589 },
		["Smite"] = { school = "Holy", id = 585 },
	},
	["ROGUE"] = {},
	["SHAMAN"] = {
		["Flametongue Weapon"] = { school = "Fire", id = 8024 },
		["Frostbrand Weapon"] = { school = "Frost", id = 8033 },
		["Lightning Bolt"] = { school = "Nature", id = 403 },
	},
	["WARLOCK"] = {
		["Incinerate"] = { school = "Fire", id = 29722 },
		["Shadow Bolt"] = { school = "Shadow", id = 686 },
		["Soul Fire"] = { school = "Fire", id = 6353 },
	 },
	["WARRIOR"] = {},
}

local spellTicks = {
	["Astral Storm"] = { id = 106996, ticks = 10 },
	["Hurricane"] = { id = 16914, ticks = 10 },
	["Tranquility"] = { id = 740, ticks = 4 },
	["Arcane Missiles"] = { id = 5143, ticks = 5 },
	["Blizzard"] = { id = 10, ticks = 8 },
	["Evocation"] = { id = 12051, ticks = 3 },
	["Drain Soul"] = { id = 1120, ticks = 6 }, 
	["Drain Life"] = { id = 689, ticks = 6 },
	["Harvest Life"] = { id = 108371, ticks = 6 },
	["Malefic Grasp"] = { id = 103103, ticks = 4 },
	["Mind Flay"] = { id = 15407, ticks = 3 },
	["Mind Sear"] = { id = 48045, ticks = 5 },
	["Penance"] = { id = 47540, ticks = 2 },
	["Soothing Mist"] = { id = 115175, ticks = 8 },
}

local classSpells = { -- table of buffs or equivalents that should be on player depending on class and specialization
	["DEATHKNIGHT"] = {
		[48266] = { 48266, 48263, 48265 }, -- "Frost Presence", "Blood Presence", "Unholy Presence"
	},
	["DRUID"] = {
		[1126] = { 1126, 20217, 115921, 90363 }, -- "Mark of the Wild", "Blessing of Kings", "Legacy of the Emperor", "Embrace of the Shale Spider"
		[24858] = { 24858 }, -- "Moonkin Form"
	},
	["HUNTER"] = {
		[13165] = { 5118, 82661, 13165, 13159, 61648 }, -- "Aspect of the Cheetah", ".. Fox", "... Hawk", "... Pack", "... Beast"
	},
	["MAGE"] = {
		[1459] = { 109773, 1459, 61316, 77747, 126309 }, -- "Dark Intent", "Arcane/Dalaran Brilliance", "Burning Wrath", "Still Water"
		[7302] = { 7302, 6117, 30482 }, -- "Frost Armor", "Mage Armor", "Molten Armor"
	},
	["MONK"] = {
		[115921] = { 1126, 20217, 115921, 90363 }, -- "Mark of the Wild", "Blessing of Kings", "Legacy of the Emperor", "Embrace of the Shale Spider"
		[103985] = { 103985, 115069, 115070 }, -- "Stance of the Fierce Tiger", "... Sturdy Ox", "... Wise Serpent"
		[116781] = { 116781, 1459, 61316, 17007, 97229, 24604, 90309, 126373, 126309 }, -- "Legacy of the White Tiger", "Arcane/Dalaran Brilliance", "Leader of the Pack", pets
	},
	["PALADIN"] = {
		[105361] = { 105361, 20165, 20164, 20154, 31801 }, -- "Seal of Command", "Seal of Insight", "Seal of Justice", "Seal of Righteousness", "Seal of Truth"
		[20217] = { 1126, 20217, 115921, 90363 }, -- "Mark of the Wild", "Blessing of Kings", "Legacy of the Emperor", "Embrace of the Shale Spider"
		[19740] = { 19740, 116956, 93435, 128997 }, -- "Grace of Air", "Blessing of Might", pets
		["exclusive"] = { 19740, 20217 }, -- "Blessing of Might", "Blessing of Kings"
	},
	["PRIEST"] = {
		[21562] = { 469, 21562, 6307, 90364 }, -- "Commanding Shout", "Power Word: Fortitude", "Blood Pact", "Qiraji Fortitude"
		[588] = { 588, 73413 },  -- "Inner Fire" or "Inner Will"
		[15473] = { 15473 }, -- "Shadowform"
		[81206] = { 81209, 81206, 81208 }, -- "Chakra: Sanctuary" or "Serenity" or "Chastise"
	},
	["ROGUE"] = {
		[2823] = { 2823, 8679 }, -- "Deadly Poison", "Wound Poison"
		[3408] = { 3408, 5761, 108215, 108211 }, -- "Crippling Poison", "Mind-numbing Poison", "Paralytic Poison", "Leeching Poison"
	},
	["SHAMAN"] = {
		[324] = { 324, 52127, 974 }, -- "Lightning Shield", "Water Shield", "Earth Shield"
		[8017] = { 8017, 8033, 8024, 51730, 8232 }, -- "Rockbiter Weapon", "Frostbrand Weapon", "Flametongue Weapon", "Earthliving Weapon", "Windfury Weapon"
	},
	["WARLOCK"] = {
		[109773] = { 109773, 1459, 61316, 77747, 126309 }, -- "Dark Intent", "Arcane/Dalaran Brilliance", "Burning Wrath", "Still Water"
	},
	["WARRIOR"] = {
		[2457] = { 2457, 71, 2458 }, -- "Battle Stance", "Defensive Stance", "Berserker Stance"
		[6673] = { 57330, 8076, 6673, 93435 }, -- "Horn of Winter", "Strength of Earth", "Battle Shout", "Roar of Courage"
		[469] = { 469, 21562, 6307, 90364 }, -- "Commanding Shout", "Power Word: Fortitude", "Blood Pact", "Qiraji Fortitude"
		["exclusive"] = { 6673, 469 }, -- "Battle Shout", "Commanding Shout"
	},
}

function Model_Spells.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, "Puffin.Model_Spells: invalid reference to data management component")
	DB.AllocateDatabase("spells", false)
	
	for k, p in pairs(runeSpells) do if p.id then p.name = GetSpellInfo(p.id); if p.name then localSpells[p.name] = k end end end
	for k, p in pairs(lockSpells[Puffin.myClass]) do if p.id then p.name = GetSpellInfo(p.id) end end
	for k, p in pairs(spellTicks) do if p.id then p.name = GetSpellInfo(p.id); if p.name then localSpells[p.name] = k end end end

	DB.SetRedirect("spells", "runes", runeSpells)
	DB.SetRedirect("spells", "local", localSpells)
	DB.SetRedirect("spells", "locks", lockSpells[Puffin.myClass])
	DB.SetRedirect("spells", "ticks", spellTicks)
	DB.SetRedirect("spells", "class", classSpells)
end

Puffin.RegisterComponent("Model_Spells", Model_Spells, "MODEL", 1)
